#include "tagfun.hh"
#include "strfun.hh"

const AttrMatchResult DoAttrMatching(const TagDef& def, const Tag& tag)
{
    AttrMatchResult result;
    /* Set default values first. */
    for(unsigned a=0; a<def.AttrList.size(); ++a)
    {
        const std::pair<std::string,std::string>& attr = def.AttrList[a];
        if(!attr.second.empty())
            result.matched[attr.first] = attr.second;
    }
    /* Now check what was given. */
    for(unsigned a=0; a<tag.Params.size(); ++a)
    {
        /* "a"="b" */
        const std::string& key   = UCase(tag.Params[a].first);
        const std::string& value = tag.Params[a].second;
        
        if(!key.empty())
        {
            result.matched[key] = value;
            //fprintf(stderr, "Adding '%s' as '%s'\n", value.c_str(), key.c_str());
        }
        else
        {
            /* just "a" */
            /* It can be a positional attribute. */
            /* If "value" is empty, use the default value instead. */
            if(value.empty()) continue;
            if(a < def.AttrList.size())
            {
                result.matched[def.AttrList[a].first] = value;
                //fprintf(stderr, "Adding '%s' as '%s' (posi)\n",
                //    value.c_str(), def.AttrList[a].first.c_str());
            }
            else
            {
                result.unmatched.push_back(value);
                //fprintf(stderr, "Adding '%s' as raw\n",value.c_str());
            }
        }
    }
    return result;
}

const std::vector<std::pair<std::string/*name*/, std::string/*value*/> >
    ParseAttrList(const std::string& s, bool default_is_name)
{
    std::vector<std::pair<std::string/*name*/, std::string/*value*/> > result;
    
    std::vector<std::string> tokens;
    std::string::size_type tagbegin=0;
    bool tagmode=false;
    char quotemode=0;
    for(std::string::size_type a=0; a<s.size(); ++a)
    {
        char ch = s[a];
        if(!quotemode && std::isspace(ch))
            { if(tagmode) tokens.push_back(s.substr(tagbegin,a-tagbegin));
              tagmode=false; continue; }
        if(!tagmode) { tagmode=true; tagbegin=a; }
        if(quotemode && ch==quotemode) { quotemode=false; continue; }
        if(!quotemode && (ch=='"' || ch=='\'')) { quotemode=ch; continue; }
    }
    if(tagmode) tokens.push_back(s.substr(tagbegin));
    for(std::string::size_type a=0; a<tokens.size(); ++a)
    {
        const std::string& token = tokens[a];
        
        //fprintf(stderr, "token: '%s'\n", token.c_str());
        
        std::string::size_type eqpos = token.find('=');
        std::string::size_type q1pos = token.find('\'');
        std::string::size_type q2pos = token.find('"');
        if(eqpos != token.npos)
        {
            if(q1pos != token.npos && q1pos < eqpos) eqpos=token.npos;
            if(q2pos != token.npos && q2pos < eqpos) eqpos=token.npos;
        }
        if(eqpos != token.npos)
            result.push_back(std::make_pair(UCase(token.substr(0, eqpos)),
                                            UnQuote(token.substr(eqpos+1))));
        else if(default_is_name)
            result.push_back(std::make_pair(UCase(token), ""));
        else
            result.push_back(std::make_pair("", UnQuote(token)));
    }
    return result;
}
